#include <stdlib.h>
#include "forms.h"
#include "main_panel.h"
#include "setup_panel.h"
#include "playlist.h"
#include "about_panel.h"
#include "engine.h"

void show_help(void)
{
	printf("\n\nMod 4 X  --  version 0.2.0\n");
	printf("Usage : Mod4X <switches> blabla.mod <more switches> foo.mod ...\n");
	printf("The following command line switches are defined in current version :\n");
	printf("   -f xxxxx	: Set playing frequency to xxxxx\n");
	printf("   -s[+-] 	: Set stereo on/off - default on\n");
	printf("   -b[+-]	: Set 16/8 bits playback - default 16 bits\n");
	printf("   -i[+-]	: Set interpolation on/off - default on\n");
	printf("   -l xxxxx	: Load playlist xxxxx\n");
	printf("   -P   	: Start playing immediately\n");
	printf("   -h    	: Show this help\n\n\n");
}

int command_line(int argc, char *argv[])
{
	int	i, j, x;
	char	s[256];
	
	/*printf("%d parameters on command line\n", argc);*/
	if ( argc == 1 ) return 0;
	for ( i=1 ; i<argc ; i++ )
	{
		/*printf("Parameter %d is %s\n", i, argv[i]);*/
		if ( argv[i][0] == '-' )
			switch ( argv[i][1] )
			{
				case 'f' : if ( (i+1) == argc ) 
					   {
					   	show_help();
					   	return 1;
					   } else
					   {
					   	i++;
					   	x = strtol(argv[i], NULL, 10);
					   	config->frequency = x;
					   }
					   break;   
				case 's' : if ( (argv[i][2] == '+') || (argv[i][2] == '\0') ) config->flags |= PF_STEREO;
					   else if ( argv[i][2] == '-' ) config->flags &= ~PF_STEREO;
					   	else
					   	{
					   		show_help();
					   		return 1;
					   	}
					   break;
				case 'i' : if ( (argv[i][2] == '+') || (argv[i][2] == '\0') ) config->flags |= PF_INTERPOLATION;
					   else if ( argv[i][2] == '-' ) config->flags &= ~PF_INTERPOLATION;
					   	else
					   	{
					   		show_help();
					   		return 1;
					   	}
					   break;
				case 'b' : if ( (argv[i][2] == '+') || (argv[i][2] == '\0') ) config->flags |= PF_16BITS;
					   else if ( argv[i][2] == '-' ) config->flags &= ~PF_16BITS;
					   	else
					   	{
					   		show_help();
					   		return 1;
					   	}
					   break;
				case 'h' : show_help();
					   return 1;
					   break;
				case 'P' :
				case 'p' : config->flags = config->flags | PF_PLAY;
					   break;
				case 'l' : if ( (i+1) == argc )
					   {
					   	show_help();
					   	return 1;
					   } else
					   {
					   	i++;
					   	load_list(argv[i]);
					   	list->updated = 1;
					   }
					   break;
			}
		else
		{
			fl_addto_browser(list->list_browser, argv[i]);
			list->updated = 1;
		}
	}
	return 0; 
} 

int main(int argc, char *argv[])
{
	/* register module loaders */
	
	ML_RegisterLoader(&load_m15);
	ML_RegisterLoader(&load_mod);
	ML_RegisterLoader(&load_mtm);
	ML_RegisterLoader(&load_s3m);
	ML_RegisterLoader(&load_stm);
	ML_RegisterLoader(&load_ult);
	ML_RegisterLoader(&load_xm);
	ML_RegisterLoader(&load_uni);
	printf("Loader(s) initialised\n");
	
	/* register drivers */
	
	MD_RegisterDriver(&drv_vox);
	printf("Driver(s) initialised\n");  

	mixer_pid = 0;
	
	fl_initialize(&argc, argv, 0, 0, 0);
	panel = create_form_main_panel();
	printf("Main panel created\n");
	
	list = create_form_playlist();
	list->active = 0;
	strcpy(list->list_name, "No_Name.lst");
	fl_set_object_label(list->list_text, list->list_name);
	printf("Playlist panel created\n");
	
	setup = create_form_setup_panel();
	setup->active = 0;
	config = (play_parameters *)malloc(sizeof(play_parameters));
	strcpy(config_file, getenv("HOME"));
	strcat(config_file, "/.Mod4Xrc");
	printf("Reading config file %s ... ", config_file);
	if ( load_parameters(config, config_file)) 
	{
		printf("failed, loading defaults.\n");
		load_defaults(config);
	} else printf("OK.\n");
	printf("Setup panel created\n");
	
	printf("Parsing command line ...\n");
	if ( command_line(argc, argv) ) return 1;
	if ( list->updated ) refresh();
	set_parameters(config);
	
	about = create_form_about_panel();	
	about->active = 0;
	printf("Info panel created\n");

	setup_engine();
	
	printf("Playing engine initialised\n");
	
	/* show the first form */
	
	fl_show_form(panel->main_panel, FL_PLACE_CENTER, 
		     		FL_FULLBORDER, "Mod 4 X -- Version 0.2.0 BETA");
	if ( config->flags & PF_PLAY ) play();
	while( fl_do_forms() != panel->quit_button);
	if ( mixer_pid )
		if ( kill(mixer_pid, 15) ) printf("Cannot terminate mixer process\n");
	return 0;
}
